@@ -29,6 +29,28 @@ make.geosse.split <- function(tree, states, nodes, split.t=Inf,
   ll
 }
 
+make.geosse.uneven <- function(tree, states, nodes, split.t=Inf,
+                             sampling.f=NULL, strict=TRUE,
+                             control=list()) {
+ cache <- make.cache.geosse.split(tree, states, nodes, split.t,
+                                 sampling.f, strict)
+ cache$info <- update.info.uneven(cache$info, make.info.geosse(tree))
+ n.part <- cache$n.part
+
+ all.branches <- make.all.branches.split.dtlik(cache, control,
+                                               initial.conditions.geosse)
+ rootfunc <- make.rootfunc.split(cache, rootfunc.geosse)
+
+ ll <- function(pars, condition.surv=TRUE, root=ROOT.OBS,
+                root.p=NULL, intermediates=FALSE) {
+   pars <- rep(list(check.pars.geosse(pars)), n.part)
+   ans <- all.branches(pars, intermediates)
+   rootfunc(ans, pars, condition.surv, root, root.p, intermediates)
+ }
+ class(ll) <- c("geosse.uneven", "geosse", "dtlik", "function")
+ ll
+}
+
 ## Make requires the usual functions:
 ## 2: make.cache (initial.tip, root)
 make.cache.geosse.split <- function(tree, states, nodes, split.t=Inf,
